@@ -1220,6 +1220,9 @@ void emit_trip(oprtype *opr, boolean_t val_output, uint4 generic_inst, int trg_r
 					ocnt_ref_seen = TRUE;
 					ocnt_ref_opr = opr;
 					break;
+				default:
+					assert(FALSE && opr->oprclass);
+					break;						/* NOTREACHED */
 			}
 			if (!inst_emitted) {
 				NON_RISC_ONLY(IGEN_GENERIC_REG(generic_inst, trg_reg))
@@ -1614,7 +1617,7 @@ void emit_trip(oprtype *opr, boolean_t val_output, uint4 generic_inst, int trg_r
  */
 int get_arg_reg(void)
 {
-	int	arg_reg_i;
+	int	arg_reg_i = GTM_REG_ACCUM; /* Default never used due to the assert in the switch below */
 
 	switch (cg_phase)
 	{
