@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -125,10 +125,10 @@ LITREF	mval	literal_batch;
 #define SKIP_ASSERT_TRUE	TRUE
 #define SKIP_ASSERT_FALSE	FALSE
 
-#define	GOTO_RETRY(SKIP_ASSERT)					\
-{								\
-	assert((CDB_STAGNATE > t_tries) || SKIP_ASSERT);	\
-	goto retry;						\
+#define	GOTO_RETRY(CDB_STATUS, SKIP_ASSERT)							\
+{												\
+	assert((CDB_STAGNATE > t_tries) || IS_FINAL_RETRY_CODE(CDB_STATUS) || SKIP_ASSERT);	\
+	goto retry;										\
 }
 
 DEFINE_NSB_CONDITION_HANDLER(gvcst_kill_ch)
@@ -279,7 +279,7 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 		trigparms.ztvalue_new = NULL;
 	)
 	operation = (do_subtree ? JNL_KILL : JNL_ZKILL);
-	for (;;)
+	for ( ; ; )
 	{
 		actual_update = 0;
 #		ifdef GTM_TRIGGER
@@ -307,11 +307,10 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 		if (cdb_sc_normal != cdb_status)
 		{	/* gvcst_root_search invoked from REDO_ROOT_SEARCH_IF_NEEDED ended up with a restart situation but did not
 			 * actually invoke t_retry. Instead, it returned control back to us asking us to restart.
+			 * Cannot enable assert (which has an assert about t_tries < CDB_STAGNATE) because it is possible for us
+			 * to get cdb_sc_gvtrootmod2 restart when t_tries == CDB_STAGNATE. Pass GOTO_RETRY parameter accordingly.
 			 */
-			GOTO_RETRY(SKIP_ASSERT_TRUE); /* cannot enable assert (which has an assert about t_tries < CDB_STAGNATE)
-						       * because it is possible for us to get cdb_sc_gvtrootmod2 restart when
-						       * t_tries == CDB_STAGNATE.
-						       */
+			GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 		}
 #		endif
 		/* Need to reinitialize gvt_hist & alt_hist for each try as it might have got set to a value in the previous
@@ -358,12 +357,12 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 					 * Needed as t_retry only resets clue of gv_target which is not the directory tree anymore.
 					 */
 					csa->dir_tree->clue.end = 0;
-					GOTO_RETRY(SKIP_ASSERT_FALSE);
+					GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 				}
 				if ((gv_altkey->end + 1) == dir_hist->h[0].curr_rec.match)
 				{	/* Case (2b) : GVT now exists for this global */
 					cdb_status = cdb_sc_gvtrootmod;
-					GOTO_RETRY(SKIP_ASSERT_FALSE);
+					GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 				} else
 				{	/* Case (2a) : GVT does not exist for this global */
 					gvt_hist = dir_hist;	/* validate directory tree history in t_end/tp_hist */
@@ -387,7 +386,7 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 			dlr_data = DG_DATAGET; /* tell dataget we want full info regarding descendants */
 			cdb_status = gvcst_dataget(&dlr_data, ztold_mval);
 			if (cdb_sc_normal != cdb_status)
-				GOTO_RETRY(SKIP_ASSERT_FALSE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 			assert((11 >= dlr_data) && (1 >= (dlr_data % 10)));
 			/* Invoke triggers for KILL as long as $data is nonzero (1 or 10 or 11).
 			 * Invoke triggers for ZKILL only if $data is 1 or 11 (for 10 case, ZKILL is a no-op).
@@ -427,7 +426,9 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 					assert(lcl_implicit_tstart || *span_status);
 					cdb_status = cdb_sc_normal;	/* signal "retry:" to avoid t_retry call */
 					assert(CDB_STAGNATE >= t_tries);
-					GOTO_RETRY(SKIP_ASSERT_TRUE);;	/* Cannot check assert because above assert is >= t_tries */
+					GOTO_RETRY(cdb_status, SKIP_ASSERT_TRUE);	/* Need to skip assert because t_tries
+											 * can be == CDB_STAGNATE.
+											 */
 				}
 				REMOVE_ZTWORM_JFB_IF_NEEDED(ztworm_jfb, jfb, sgm_info_ptr);
 			}
@@ -463,11 +464,11 @@ research:
 				/* Skip assert inside GOTO_RETRY macro as the WBTEST_ANTIFREEZE_GVKILLFAIL white-box testcase
 				 * intentionally triggers a GVKILLFAIL error.
 				 */
-				GOTO_RETRY(SKIP_ASSERT_TRUE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_TRUE);
 			}
 #endif
 			if (cdb_sc_normal != (cdb_status = gvcst_search(gv_currkey, NULL)))
-				GOTO_RETRY(SKIP_ASSERT_FALSE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 			assert(gv_altkey->top == gv_currkey->top);
 			assert(gv_altkey->top == gv_keysize);
 			end = gv_currkey->end;
@@ -511,7 +512,7 @@ research:
 						{
 							cdb_status = tp_hist(NULL);
 							if (cdb_sc_normal != cdb_status)
-								GOTO_RETRY(SKIP_ASSERT_FALSE);
+								GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 							*span_status = TRUE;
 #							ifdef GTM_TRIGGER
 							if (lcl_implicit_tstart)
@@ -542,11 +543,11 @@ research:
 			}
 			gv_altkey->end = end;
 			if (cdb_sc_normal != (cdb_status = gvcst_search(gv_altkey, alt_hist)))
-				GOTO_RETRY(SKIP_ASSERT_FALSE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 			if (alt_hist->depth != gvt_hist->depth)
 			{
 				cdb_status = cdb_sc_badlvl;
-				GOTO_RETRY(SKIP_ASSERT_FALSE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 			}
 			right_extra = FALSE;
 			left_extra = TRUE;
@@ -574,7 +575,7 @@ research:
 						goto research;
 					}
 					if (cdb_sc_delete_parent != cdb_status)
-						GOTO_RETRY(SKIP_ASSERT_FALSE);
+						GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 					left_extra = right_extra
 						   = TRUE;
 				} else
@@ -601,7 +602,7 @@ research:
 						left_extra = TRUE;
 						cdb_status = cdb_sc_normal;
 					} else
-						GOTO_RETRY(SKIP_ASSERT_FALSE);
+						GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 					local_srch_rec.offset = local_srch_rec.match
 							      = 0;
 					cdb_status = gvcst_kill_blk(right, lev, gv_altkey, local_srch_rec, right->curr_rec,
@@ -617,7 +618,7 @@ research:
 						right_extra = TRUE;
 						cdb_status = cdb_sc_normal;
 					} else
-						GOTO_RETRY(SKIP_ASSERT_FALSE);
+						GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 				}
 			}
 		}
@@ -784,7 +785,7 @@ research:
                 {
                         cdb_status = tp_hist(alt_hist);
                         if (cdb_sc_normal != cdb_status)
-				GOTO_RETRY(SKIP_ASSERT_FALSE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
                 }
 		/* Note down $tlevel (used later) before it is potentially changed by op_tcommit below */
 		lcl_dollar_tlevel = dollar_tlevel;
@@ -794,7 +795,7 @@ research:
 			assert(gvt_root);
 			GVTR_OP_TCOMMIT(cdb_status);
 			if (cdb_sc_normal != cdb_status)
-				GOTO_RETRY(SKIP_ASSERT_TRUE);
+				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 		}
 #		endif
 		if (!killing_chunks)
