@@ -23,7 +23,7 @@
 # include <ssdef.h>
 #endif
 #ifdef UNIX
-# include <signal.h>
+#include "gtm_signal.h"
 # include <stdarg.h>
 #endif
 
@@ -622,7 +622,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 		TREF(in_op_fnnext) = FALSE;			/* in case we were in $NEXT */
 		/* Global values that may need cleanups */
 		if (INTRPT_OK_TO_INTERRUPT != intrpt_ok_state)
-			ENABLE_INTERRUPTS(intrpt_ok_state);	/* If interrupts were deferred, re-enable them now */
+			ENABLE_INTERRUPTS(intrpt_ok_state, INTRPT_OK_TO_INTERRUPT);	/* If interrupts were deferred,
+											 * re-enable them now */
 	}
 #	ifdef GTM_TRIGGER
 	/* At this point, we are past the point where the frame pointer is allowed to be resting on a trigger frame
@@ -686,9 +687,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 				 frame_pointer->type));
 			frame_pointer->mpc = restart_pc;
 			frame_pointer->ctxt = restart_ctxt;
-			frame_pointer->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
+			frame_pointer->flags &= SSF_NORET_VIA_MUMTSTART_OFF;	/* Frame enterable now with mpc reset */
 			GTMTRIG_ONLY(
-				DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD_OFF (1) in frame "
+				DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART_OFF (1) in frame "
 					  "0x"lvaddr"\n", frame_pointer)));
 			if (!(frame_pointer->type & SFT_DM))
 				dm_setup();
@@ -696,9 +697,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 		{
 			frame_pointer->ctxt = GTM_CONTEXT(call_dm);
 			frame_pointer->mpc = CODE_ADDRESS(call_dm);
-			frame_pointer->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
+			frame_pointer->flags &= SSF_NORET_VIA_MUMTSTART_OFF;	/* Frame enterable now with mpc reset */
 			GTMTRIG_ONLY(
-				DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD_OFF (1) in frame "
+				DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART_OFF (1) in frame "
 					  "0x"lvaddr"\n", frame_pointer)));
 		} else
 		{
@@ -706,9 +707,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 			IF_INDR_FRAME_CLEANUP_CACHE_ENTRY_AND_UNMARK(frame_pointer);
 			frame_pointer->ctxt = GTM_CONTEXT(pseudo_ret);
 			frame_pointer->mpc = CODE_ADDRESS(pseudo_ret);
-			frame_pointer->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
+			frame_pointer->flags &= SSF_NORET_VIA_MUMTSTART_OFF;	/* Frame enterable now with mpc reset */
 			GTMTRIG_ONLY(
-				DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD_OFF (1) in frame "
+				DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART_OFF (1) in frame "
 					  "0x"lvaddr"\n", frame_pointer)));
 		}
 		PRN_ERROR;
@@ -820,8 +821,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 		{
 			frame_pointer->ctxt = GTM_CONTEXT(call_dm);
 			frame_pointer->mpc = CODE_ADDRESS(call_dm);
-			frame_pointer->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
-			GTMTRIG_ONLY(DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD (2) in frame 0x"
+			frame_pointer->flags &= SSF_NORET_VIA_MUMTSTART_OFF;	/* Frame enterable now with mpc reset */
+			GTMTRIG_ONLY(DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART (2) in frame 0x"
 					       lvaddr"\n", frame_pointer)));
 		} else
 		{
@@ -829,8 +830,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 			IF_INDR_FRAME_CLEANUP_CACHE_ENTRY_AND_UNMARK(frame_pointer);
 			frame_pointer->ctxt = GTM_CONTEXT(pseudo_ret);
 			frame_pointer->mpc = CODE_ADDRESS(pseudo_ret);
-			frame_pointer->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
-			GTMTRIG_ONLY(DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD (3) in frame 0x"
+			frame_pointer->flags &= SSF_NORET_VIA_MUMTSTART_OFF;	/* Frame enterable now with mpc reset */
+			GTMTRIG_ONLY(DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART (3) in frame 0x"
 					       lvaddr"\n", frame_pointer)));
 		}
 		PRN_ERROR;
@@ -1012,9 +1013,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 					{
 						assert(SFF_INDCE & fp->flags);
 						fp->mpc = fp->ctxt;
-						fp->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
+						fp->flags &= SSF_NORET_VIA_MUMTSTART_OFF; /* Frame enterable now with mpc reset */
 						GTMTRIG_ONLY(
-							DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD"
+							DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART"
 								  " (4) in frame 0x"lvaddr"\n", frame_pointer)));
 						break;
 					}
@@ -1025,18 +1026,18 @@ CONDITION_HANDLER(mdb_condition_handler)
 					{	/* GT.M specific error trapping retries the line with the error */
 						fp->mpc = dollar_ecode.error_last_b_line;
 						fp->ctxt = context;
-						fp->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
+						fp->flags &= SSF_NORET_VIA_MUMTSTART_OFF; /* Frame enterable now with mpc reset */
 						GTMTRIG_ONLY(
-							DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD"
+							DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART"
 								  " (5) in frame 0x"lvaddr"\n", frame_pointer)));
 						break;
 					} else
 					{
 						fp->ctxt = GTM_CONTEXT(pseudo_ret);
 						fp->mpc = CODE_ADDRESS(pseudo_ret);
-						fp->flags &= SFF_IMPLTSTART_CALLD_OFF;	/* Frame enterable now with mpc reset */
+						fp->flags &= SSF_NORET_VIA_MUMTSTART_OFF; /* Frame enterable now with mpc reset */
 						GTMTRIG_ONLY(
-							DBGTRIGR((stderr, "mdb_condition_handler: turning off SFF_IMPLTSTART_CALLD"
+							DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART"
 							  " (6) in frame 0x"lvaddr"\n", frame_pointer)));
 					}
 				}
