@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -50,55 +51,47 @@
 #include "mupint.h"
 #include "mu_gv_cur_reg_init.h"
 
-#define DUMMY_GLOBAL_VARIABLE   "%D%DUMMY_VARIABLE"
-#define DUMMY_GLOBAL_VARIABLE_LEN SIZEOF(DUMMY_GLOBAL_VARIABLE)
-#define MAX_UTIL_LEN 80
-#define APPROX_ALL_ERRORS 1000000
-#define DEFAULT_ERR_LIMIT 10
-#define PERCENT_FACTOR 100
-#define PERCENT_DECIMAL_SCALE 100000
-#define PERCENT_SCALE_FACTOR 1000
-#define LEAVE_BLOCKS_ALONE 0x0FFFFFFFFUL
+#define DUMMY_GLOBAL_VARIABLE		"%D%DUMMY_VARIABLE"
+#define DUMMY_GLOBAL_VARIABLE_LEN	SIZEOF(DUMMY_GLOBAL_VARIABLE)
+#define MAX_UTIL_LEN			96
+#define APPROX_ALL_ERRORS		1000000
+#define DEFAULT_ERR_LIMIT		10
+#define PERCENT_FACTOR			100
+#define PERCENT_DECIMAL_SCALE		100000
+#define PERCENT_SCALE_FACTOR		1000
 
 #define TEXT1 " is incorrect, should be "
 #define TEXT2 "!/Largest transaction number found in database was "
 #define TEXT3 "Current transaction number is                    "
 #define MSG1  "!/!/WARNING: Transaction number reset complete on all active blocks. Please do a DATABASE BACKUP before proceeding"
-/* The QWPERCENTCALC calculates the percent used compatibly with the code fragment below.
- * The size is no longer int_size but is now a qw_num, so the QW macros are now used to
- * do the calculation.
- *
- * leftpt  = (int_size * PERCENT_FACTOR) / (int_blks * blk_size);
- * rightpt = (int_size * PERCENT_DECIMAL_SCALE) / (int_blks * blk_size) - (leftpt * PERCENT_SCALE_FACTOR);
- */
+/* The QWPERCENTCALC calculates the percent used in two scaled parts*/
 
-#define QWPERCENTCALC(lpt, rpt, qwint_size, int_blks, blk_size) 	\
-{									\
-	qw_num	tmp;							\
-	size_t	rem; 						\
-									\
-	if (int_blks)							\
-	{								\
-		QWMULBYDW(tmp, (qwint_size), PERCENT_DECIMAL_SCALE);	\
-		QWDIVIDEBYDW(tmp, (int_blks), tmp, rem);		\
-		QWDIVIDEBYDW(tmp, (blk_size), tmp, rem);		\
-		QWDIVIDEBYDW(tmp, PERCENT_SCALE_FACTOR, tmp, rpt);	\
-		DWASSIGNQW((lpt), tmp);					\
-	} else								\
-	{								\
-		(lpt) = 0;						\
-		(rpt) = 0;						\
-	}								\
+#define QWPERCENTCALC(lpt, rpt, sizes, int_blks, blk_size) 									\
+{																\
+	if (int_blks)														\
+	{															\
+		lpt = ((sizes) * PERCENT_FACTOR) / ((int_blks) * (blk_size));							\
+		rpt = ((((sizes) * PERCENT_DECIMAL_SCALE) / ((int_blks) * (blk_size))) - (lpt * PERCENT_SCALE_FACTOR));		\
+	} else															\
+		lpt = rpt = 0;													\
+}
+
+#define CUMULATE_TOTAL(T_TYPE, IDX)												\
+{																\
+	for (c_type = BLKS; c_type < CUM_TYPE_MAX; c_type++)									\
+	{															\
+		GTM_WHITE_BOX_TEST(WBTEST_FAKE_BIG_CNTS, mu_int_cum[c_type][IDX], (mu_int_cum[c_type][IDX] << 31));		\
+		mu_int_tot[T_TYPE][c_type] += mu_int_cum[c_type][IDX];								\
+		mu_int_cum[c_type][IDX] = 0;											\
+	}															\
 }
 
 GBLDEF unsigned char		mu_int_root_level;
-GBLDEF int4			mu_int_adj[MAX_BT_DEPTH + 1];
+GBLDEF uint4			mu_int_adj[MAX_BT_DEPTH + 1];
 GBLDEF uint4			mu_int_errknt;
-GBLDEF uint4			mu_int_skipreg_cnt=0;
-GBLDEF uint4			mu_int_blks[MAX_BT_DEPTH + 1];
 GBLDEF uint4			mu_int_offset[MAX_BT_DEPTH + 1];
-GBLDEF uint4			mu_int_recs[MAX_BT_DEPTH + 1];
-GBLDEF qw_num			mu_int_size[MAX_BT_DEPTH + 1];
+GBLDEF uint4			mu_int_skipreg_cnt = 0;
+GBLDEF gtm_uint64_t		mu_int_cum[CUM_TYPE_MAX][MAX_BT_DEPTH + 1];
 GBLDEF int			disp_map_errors;
 GBLDEF int			disp_maxkey_errors;
 GBLDEF int			disp_trans_errors;
@@ -124,11 +117,10 @@ GBLDEF unsigned char		*mu_int_master;
 GBLDEF trans_num		largest_tn;
 GBLDEF int4			mu_int_blks_to_upgrd;
 GBLDEF span_node_integ		*sndata;
-/* The following global variable is used to store the encryption information for the current database. The
- * variable is initialized in mu_int_init(mupip integ -file <file.dat>) and mu_int_reg(mupip integ -reg <reg_name>). */
-GTMCRYPT_ONLY(
-	GBLDEF	gtmcrypt_key_t	mu_int_encrypt_key_handle;
-)
+/* The following global variable is used to store the encryption information for the current database. The variable is initialized
+ * in mu_int_init (mupip integ -file <file.dat>) and mu_int_reg (mupip integ -reg <reg_name>).
+ */
+GBLDEF enc_handles		mu_int_encr_handles;
 GBLDEF boolean_t		ointeg_this_reg;
 GTM_SNAPSHOT_ONLY(
 	GBLDEF util_snapshot_ptr_t	util_ss_ptr;
@@ -173,30 +165,30 @@ error_def(ERR_DBSPANCHUNKORD);
 
 void mupip_integ(void)
 {
-	boolean_t		full, muint_all_index_blocks;
+	boolean_t		full, muint_all_index_blocks, retvalue_mu_int_reg, region_was_frozen;
 	boolean_t		update_filehdr, update_header_tn;
+	boolean_t		online_integ = FALSE;
 	char			*temp, util_buff[MAX_UTIL_LEN];
 	unsigned char		dummy;
 	unsigned char		key_buff[2048];
 	short			iosb[4];
 	unsigned short		keylen;
-	unsigned int		blocks_free = LEAVE_BLOCKS_ALONE;
-	int			idx, leftpt, rightpt, total_errors, util_len;
-	uint4			cli_status;
-	block_id		dir_root, mu_index_adj, mu_data_adj, muint_block;
-	uint4			prev_errknt, mu_dir_blks, mu_dir_recs, mu_data_blks, mu_data_recs, mu_index_blks, mu_index_recs;
-	uint4			tot_blks;
-	qw_num			mu_dir_size, mu_index_size, mu_data_size;
-	tp_region		*rptr;
+	int			idx, total_errors, util_len;
+	uint4			cli_status, leftpt, mu_data_adj, mu_index_adj, prev_errknt, rightpt;
+	block_id		dir_root, muint_block;
+	enum cum_type		c_type;
+	enum tot_type		t_type;
 	file_control		*fc;
-	boolean_t		retvalue_mu_int_reg, online_integ = FALSE, region_was_frozen;
+	gtm_uint64_t		blocks_free = (gtm_uint64_t)MAXUINT8;
+	gtm_uint64_t		mu_int_tot[TOT_TYPE_MAX][CUM_TYPE_MAX], tot_blks, tot_recs;
+	tp_region		*rptr;
+	sgmnt_addrs		*csa;
+	sgmnt_data_ptr_t	csd;
+	span_node_integ		span_node_data;
 	GTM_SNAPSHOT_ONLY(
 		char		ss_filename[GTM_PATH_MAX];
 		unsigned short	ss_file_len = GTM_PATH_MAX;
 	)
-	sgmnt_addrs		*csa;
-	sgmnt_data_ptr_t	csd;
-	span_node_integ		span_node_data;
 
 	sndata = &span_node_data;
 	error_mupip = FALSE;
@@ -252,7 +244,6 @@ void mupip_integ(void)
 		muint_fast = TRUE;
 	else
 		muint_fast = FALSE;
-
 	/* DBG qualifier prints extra debug messages while waiting for KIP in region freeze */
 	debug_mupip = (CLI_PRESENT == cli_present("DBG"));
 	GTM_SNAPSHOT_ONLY(online_specified = (CLI_PRESENT == cli_present("ONLINE"));)
@@ -273,12 +264,12 @@ void mupip_integ(void)
 		gvinit(); /* side effect: initializes gv_altkey (used by code below) & gv_currkey (not used by below code) */
 		region = TRUE;
 		mu_getlst("WHAT", SIZEOF(tp_region));
-                if (!grlist)
-                {
+		if (!grlist)
+		{
 			error_mupip = TRUE;
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DBNOREGION);
 			mupip_exit(ERR_MUNOACTION);
-                }
+		}
 		rptr = grlist;
 	} else
 		GVKEY_INIT(gv_altkey, DBKEYSIZE(MAX_KEY_SZ));	/* used by code below */
@@ -341,22 +332,13 @@ void mupip_integ(void)
 				continue;
 			}
 		}
-		QWASSIGNDW(mu_dir_size, 0);
-		QWASSIGNDW(mu_index_size, 0);
-		QWASSIGNDW(mu_data_size, 0);
+		memset(mu_int_tot, 0, SIZEOF(mu_int_tot));
+		memset(mu_int_cum, 0, SIZEOF(mu_int_tot));
 		mu_index_adj = mu_data_adj = 0;
-		mu_dir_blks = mu_dir_recs = 0;
-		mu_data_blks = mu_data_recs = 0;
-		mu_index_blks = mu_index_recs = 0;
 		mu_int_err_ranges = (CLI_NEGATED != cli_present("KEYRANGES"));
 		mu_int_root_level = BML_LEVL;	/* start with what is an invalid level for a root block */
 		mu_map_errs = 0, prev_errknt = 0, largest_tn = 0;
 		mu_int_blks_to_upgrd = 0;
-		for (idx = 0;  idx <= MAX_BT_DEPTH;  idx++)
-		{
-			QWASSIGNDW(mu_int_size[idx], 0);
-			mu_int_blks[idx] = mu_int_recs[idx] = 0;
-		}
 		mu_int_path[0] = 0;
 		mu_int_offset[0] = 0;
 		mu_int_plen = 1;
@@ -407,13 +389,16 @@ void mupip_integ(void)
 		tn_reset_this_reg = update_header_tn = FALSE;
 		if (tn_reset_specified)
 		{
-			if (gv_cur_region->read_only)
+			if (gv_cur_region->read_only || (USES_NEW_KEY(csd)))
 			{
-				gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBRDONLY, 2, gv_cur_region->dyn.addr->fname_len,
-					gv_cur_region->dyn.addr->fname);
+				if (gv_cur_region->read_only)
+					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBRDONLY, 2, DB_LEN_STR(gv_cur_region));
+				else
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) MAKE_MSG_TYPE(ERR_TEXT, ERROR), 2,
+						LEN_AND_LIT("Database is being (re)encrypted"));
 				mu_int_errknt++;
 				mu_int_err(ERR_DBTNRESET, 0, 0, 0, 0, 0, 0, 0);
-				mu_int_errknt-=2;
+				mu_int_errknt -= 2;
 				/* is this error supposed to update error count, or leave it ( then mu_int_errknt-- instead)*/
 				mu_int_plen++;  /* continuing, so compensate for mu_int_err decrement */
 			} else
@@ -473,17 +458,17 @@ void mupip_integ(void)
 			}
 			mu_int_plen = 0;
 			memset(mu_int_adj_prev, 0, SIZEOF(mu_int_adj_prev));
- 			gv_target->nct = trees->nct;
- 			gv_target->act = trees->act;
- 			gv_target->ver = trees->ver;
- 			gv_altkey->prev = 0;
+			gv_target->nct = trees->nct;
+			gv_target->act = trees->act;
+			gv_target->ver = trees->ver;
+			gv_altkey->prev = 0;
 			assert(trees->keysize == strlen(trees->key));
- 			gv_altkey->end = trees->keysize;
+			gv_altkey->end = trees->keysize;
 			assert(gv_altkey->end + 2 <= gv_altkey->top);
- 			memcpy(gv_altkey->base, trees->key, gv_altkey->end);
- 			gv_altkey->base[gv_altkey->end++] = '\0';
- 			gv_altkey->base[gv_altkey->end] = '\0';
- 			if (gv_target->act)
+			memcpy(gv_altkey->base, trees->key, gv_altkey->end);
+			gv_altkey->base[gv_altkey->end++] = '\0';
+			gv_altkey->base[gv_altkey->end] = '\0';
+			if (gv_target->act)
 				act_in_gvt(gv_target);
 			if (mu_int_blk(trees->root, MAX_BT_DEPTH, TRUE, gv_altkey->base, gv_altkey->end, &dummy, 0, 0))
 			{
@@ -531,51 +516,40 @@ void mupip_integ(void)
 					for (idx = mu_int_root_level;  idx >= 0;  idx--)
 					{
 						if ((0 == idx) && muint_fast && (trees->root != dir_root))
-						util_out_print("!5UL    !12UL              NA              NA  !12UL", TRUE,
-								idx, mu_int_blks[idx],  mu_int_adj[idx]);
+						util_out_print("!5UL !15@UQ              NA              NA  !12UL", TRUE,
+								idx, &mu_int_cum[BLKS][idx],  mu_int_adj[idx]);
 						else
 						{
-							QWPERCENTCALC(leftpt, rightpt, mu_int_size[idx], mu_int_blks[idx],
-													mu_int_data.blk_size);
+							QWPERCENTCALC(leftpt, rightpt, mu_int_cum[SIZE][idx],
+								mu_int_cum[BLKS][idx], mu_int_data.blk_size);
 							if (trees->root != dir_root)
 							{
-								util_out_print("!5UL    !12UL    !12UL    !8UL.!3ZL  !12UL",
-									TRUE, idx, mu_int_blks[idx], mu_int_recs[idx],
-									leftpt, rightpt, mu_int_adj[idx]);
+								util_out_print("!5UL !15@UQ !15@UQ    !8UL.!3ZL  !12UL",
+									TRUE, idx, &mu_int_cum[BLKS][idx],
+									&mu_int_cum[RECS][idx], leftpt, rightpt, mu_int_adj[idx]);
 							} else
-								util_out_print("!5UL    !12UL    !12UL    !8UL.!3ZL           NA",
-									TRUE, idx, mu_int_blks[idx], mu_int_recs[idx],
-									leftpt, rightpt);
+								util_out_print("!5UL !15@UQ !15@UQ    !8UL.!3ZL           NA",
+									TRUE, idx, &mu_int_cum[BLKS][idx],
+									&mu_int_cum[RECS][idx], leftpt, rightpt);
 						}
 					}
 				}
 				if (dir_root == trees->root)
 				{
 					for (idx = mu_int_root_level;  idx >= 0;  idx--)
-					{
-						mu_dir_blks += mu_int_blks[idx];
-						QWINCRBY(mu_dir_size, mu_int_size[idx]);
-						mu_dir_recs += mu_int_recs[idx];
-						QWASSIGNDW(mu_int_size[idx], 0);
-						mu_int_adj[0] = mu_int_blks[idx] = mu_int_recs[idx] = 0;
-					}
+						CUMULATE_TOTAL(DIRTREE, idx);
+					mu_int_adj[0] = 0;
 				} else
 				{
 					for (idx = mu_int_root_level;  idx > 0;  idx--)
 					{
-						mu_index_blks += mu_int_blks[idx];
-						QWINCRBY(mu_index_size, mu_int_size[idx]);
-						mu_index_recs += mu_int_recs[idx];
+						CUMULATE_TOTAL(INDX, idx);
 						mu_index_adj += mu_int_adj[idx];
-						QWASSIGNDW(mu_int_size[idx], 0);
-						mu_int_adj[idx] = mu_int_blks[idx] = mu_int_recs[idx] = 0;
+						mu_int_adj[idx] = 0;
 					}
-					mu_data_blks += mu_int_blks[0];
+					CUMULATE_TOTAL(DATA, 0);
 					mu_data_adj += mu_int_adj[0];
-					QWINCRBY(mu_data_size, mu_int_size[0]);
-					mu_data_recs += mu_int_recs[0];
-					QWASSIGNDW(mu_int_size[0], 0);
-					mu_int_adj[0] = mu_int_blks[0] = mu_int_recs[0] = 0;
+					mu_int_adj[0] = 0;
 				}
 			} else  if (update_header_tn)
 			{
@@ -593,11 +567,12 @@ void mupip_integ(void)
 		if (muint_all_index_blocks)
 		{
 			mu_int_maps();
-			if (! mu_int_errknt)
-			{
-				blocks_free = mu_int_data.trans_hist.total_blks -
-					(mu_int_data.trans_hist.total_blks + mu_int_data.bplmap - 1) / mu_int_data.bplmap -
-					mu_data_blks - mu_index_blks - mu_dir_blks;
+			if (!mu_int_errknt)
+			{	/* because it messes with the totals, the white box case does not produce an accurate result */
+				blocks_free = (gtm_uint64_t)mu_int_data.trans_hist.total_blks
+					- (((gtm_uint64_t)mu_int_data.trans_hist.total_blks + (gtm_uint64_t)mu_int_data.bplmap - 1)
+					/ (gtm_uint64_t)mu_int_data.bplmap)
+					- mu_int_tot[DATA][BLKS] - mu_int_tot[INDX][BLKS] - mu_int_tot[DIRTREE][BLKS];
 				/* If ONLINE INTEG, then cs_addrs->hdr->trans_hist.free_blocks can no longer be expected to remain
 				 * the same as it was during the time INTEG started as updates are allowed when ONLINE INTEG is
 				 * in progress and hence use mu_int_data.trans_hist.free_blocks as it is the copy of the file header
@@ -609,14 +584,15 @@ void mupip_integ(void)
 						mu_int_errknt++;
 					util_len = SIZEOF("!/Free blocks counter in file header:  ") - 1;
 					memcpy(util_buff, "!/Free blocks counter in file header:  ", util_len);
-					util_len += i2hex_nofill(csd->trans_hist.free_blocks, (uchar_ptr_t)&util_buff[util_len], 8);
+					util_len += i2hexl_nofill(csd->trans_hist.free_blocks,
+							(uchar_ptr_t)&util_buff[util_len], 16);
 					MEMCPY_LIT(&util_buff[util_len], TEXT1);
 					util_len += SIZEOF(TEXT1) - 1;
-					util_len += i2hex_nofill(blocks_free, (uchar_ptr_t)&util_buff[util_len], 8);
+					util_len += i2hexl_nofill(blocks_free, (uchar_ptr_t)&util_buff[util_len], 16);
 					util_buff[util_len] = 0;
 					util_out_print(util_buff, TRUE);
 				} else
-					blocks_free = LEAVE_BLOCKS_ALONE;
+					blocks_free = (gtm_uint64_t)MAXUINT8;
 			}
 			if (!muint_fast && (mu_int_blks_to_upgrd != csd->blks_to_upgrd))
 			{
@@ -648,36 +624,43 @@ void mupip_integ(void)
 				util_out_print("!/No errors detected by integ.", TRUE);
 		}
 		util_out_print("!/Type           Blocks         Records          % Used      Adjacent!/", TRUE);
-		QWPERCENTCALC(leftpt, rightpt, mu_dir_size, mu_dir_blks, mu_int_data.blk_size);
-		util_out_print("Directory    !8UL    !12UL    !8UL.!3ZL            NA", TRUE, mu_dir_blks, mu_dir_recs,
-					leftpt, rightpt);
-		QWPERCENTCALC(leftpt, rightpt, mu_index_size, mu_index_blks, mu_int_data.blk_size);
-		util_out_print("Index    !12UL    !12UL    !8UL.!3ZL  !12UL", TRUE, mu_index_blks, mu_index_recs, leftpt, rightpt,
-			mu_index_adj);
+		QWPERCENTCALC(leftpt, rightpt, mu_int_tot[DIRTREE][SIZE], mu_int_tot[DIRTREE][BLKS], mu_int_data.blk_size);
+		util_out_print("Directory !11@UQ !15@UQ    !8UL.!3ZL            NA", TRUE, &mu_int_tot[DIRTREE][BLKS],
+			&mu_int_tot[DIRTREE][RECS], leftpt, rightpt);
+		QWPERCENTCALC(leftpt, rightpt, mu_int_tot[INDX][SIZE], mu_int_tot[INDX][BLKS], mu_int_data.blk_size);
+		util_out_print("Index !15@UQ !15@UQ    !8UL.!3ZL  !12UL", TRUE, &mu_int_tot[INDX][BLKS],
+			&mu_int_tot[INDX][RECS], leftpt, rightpt, mu_index_adj);
 		if (muint_fast)
-			util_out_print("Data     !12UL              NA              NA  !12UL", TRUE, mu_data_blks, mu_data_adj);
+			util_out_print("Data  !15@UQ              NA              NA  !12UL", TRUE,
+				&mu_int_tot[DATA][BLKS], mu_data_adj);
 		else
 		{
-			QWPERCENTCALC(leftpt, rightpt, mu_data_size, mu_data_blks, mu_int_data.blk_size);
-			util_out_print("Data     !12UL    !12UL    !8UL.!3ZL  !12UL", TRUE, mu_data_blks, mu_data_recs,
-					leftpt, rightpt, mu_data_adj);
+			QWPERCENTCALC(leftpt, rightpt, mu_int_tot[DATA][SIZE], mu_int_tot[DATA][BLKS], mu_int_data.blk_size);
+			util_out_print("Data  !15@UQ !15@UQ    !8UL.!3ZL  !12UL", TRUE, &mu_int_tot[DATA][BLKS],
+				&mu_int_tot[DATA][RECS], leftpt, rightpt, mu_data_adj);
 		}
 		if ((FALSE == block) && (MUINTKEY_FALSE == muint_key))
 		{
-			util_out_print("Free     !12UL              NA              NA            NA", TRUE,
-				mu_int_data.trans_hist.total_blks -
-				(mu_int_data.trans_hist.total_blks + mu_int_data.bplmap - 1) / mu_int_data.bplmap -
-				mu_data_blks - mu_index_blks - mu_dir_blks);
+			tot_blks = mu_int_data.trans_hist.total_blks
+				- ((mu_int_data.trans_hist.total_blks + mu_int_data.bplmap - 1) / mu_int_data.bplmap);
+			GTM_WHITE_BOX_TEST(WBTEST_FAKE_BIG_CNTS, tot_blks, tot_blks << 31);
+			tot_blks = tot_blks - mu_int_tot[DATA][BLKS] - mu_int_tot[INDX][BLKS] - mu_int_tot[DIRTREE][BLKS];
+			util_out_print("Free  !15@UQ              NA              NA            NA", TRUE, &tot_blks);
 			tot_blks = mu_int_data.trans_hist.total_blks
 					- (mu_int_data.trans_hist.total_blks + mu_int_data.bplmap - 1) / mu_int_data.bplmap;
+			GTM_WHITE_BOX_TEST(WBTEST_FAKE_BIG_CNTS, tot_blks,
+					mu_int_tot[DATA][BLKS] + mu_int_tot[INDX][BLKS] + mu_int_tot[DIRTREE][BLKS]);
 		} else
-			tot_blks = mu_data_blks + mu_index_blks + mu_dir_blks;
+			tot_blks = mu_int_tot[DATA][BLKS] + mu_int_tot[INDX][BLKS] + mu_int_tot[DIRTREE][BLKS];
 		if (muint_fast)
-			util_out_print("Total    !12UL              NA              NA  !12UL", TRUE,
-				tot_blks, mu_data_adj + mu_index_adj);
+			util_out_print("Total !15@UQ              NA              NA  !12UL", TRUE,
+				&tot_blks, mu_data_adj + mu_index_adj);
 		else
-			util_out_print("Total    !12UL    !12UL              NA  !12UL", TRUE,
-				tot_blks, mu_dir_recs + mu_index_recs + mu_data_recs, mu_data_adj + mu_index_adj);
+		{
+			tot_recs = mu_int_tot[DIRTREE][RECS] + mu_int_tot[INDX][RECS] + mu_int_tot[DATA][RECS];
+			util_out_print("Total !15@UQ !15@UQ              NA  !12UL", TRUE,
+				&tot_blks, &tot_recs, mu_data_adj + mu_index_adj);
+		}
 		if (sndata->sn_cnt)
 		{
 			util_out_print("[Spanning Nodes:!UL ; Blocks:!UL]", TRUE, sndata->sn_cnt, sndata->sn_blk_cnt);
@@ -720,7 +703,7 @@ void mupip_integ(void)
 			 */
 			if (!gv_cur_region->read_only && !ointeg_this_reg)
 			{
-				if (LEAVE_BLOCKS_ALONE != blocks_free)
+				if ((gtm_uint64_t)MAXUINT8 != blocks_free)
 					csd->trans_hist.free_blocks = blocks_free;
 				if (!mu_int_errknt && muint_all_index_blocks && !muint_fast)
 				{
@@ -766,7 +749,7 @@ void mupip_integ(void)
 					mu_int_data.kill_in_prog = 0;
 					update_filehdr = TRUE;
 				}
-				if ((LEAVE_BLOCKS_ALONE != blocks_free) && (mu_int_data.trans_hist.free_blocks != blocks_free))
+				if ((MAXUINT8 != blocks_free) && (mu_int_data.trans_hist.free_blocks != blocks_free))
 				{
 					mu_int_data.trans_hist.free_blocks = blocks_free;
 					update_filehdr = TRUE;
