@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -102,9 +102,7 @@ VMS_ONLY(static  const   unsigned short  zero_fid[3];)
 uint4	mupip_set_journal(unsigned short db_fn_len, char *db_fn)
 {
 	jnl_create_info		jnl_info;
-	GTMCRYPT_ONLY(
-		jnl_create_info	*jnl_info_ptr;
-	)
+	jnl_create_info		*jnl_info_ptr;
 	GDS_INFO		*gds_info;
 	file_control		*fc;
 	int			new_stat_res;	/* gtm_file_stat() return value for new journal file name */
@@ -300,7 +298,7 @@ uint4	mupip_set_journal(unsigned short db_fn_len, char *db_fn)
 					exit_status |= EXIT_ERR;
 					continue; /* Later mupip_set_jnl_cleanup() will do the cleanup */
 				}
-				cs_data = rptr->sd = csd;
+				cs_addrs->hdr = cs_data = rptr->sd = csd;
 				rptr->state = ALLOCATED;
 			} else
 			{
@@ -512,17 +510,8 @@ uint4	mupip_set_journal(unsigned short db_fn_len, char *db_fn)
 				break;
 			}
 			memcpy(jnl_info.jnl, tmpjnlfile.addr, tmpjnlfile.len);
-			jnl_info.jnl_len = tmpjnlfile.len;
-			jnl_info.jnl[jnl_info.jnl_len] = 0;
+			jnl_info.jnl_len = tmpjnlfile.len; /* get_full_path() null terminates and limits tmpjnlfile to MAX_FN_LEN */
 			/* Note: At this point jnlfile should have expanded journal name with extension */
-			if (MAX_FN_LEN + 1 < jnl_info.jnl_len)
-			{
-				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_FILENAMETOOLONG);
-				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_JNLNOCREATE, 2,
-						jnl_info.jnl_len, jnl_info.jnl);
-				exit_status |= EXIT_ERR;
-				break;
-			}
 			jnlname_same = ((jnl_info.jnl_len == csd->jnl_file_len)
 					&& (0 == memcmp(jnl_info.jnl, csd->jnl_file_name, jnl_info.jnl_len))) ? TRUE : FALSE;
 			jnlfile.addr = (char *)csd->jnl_file_name;
@@ -737,10 +726,8 @@ uint4	mupip_set_journal(unsigned short db_fn_len, char *db_fn)
 				jnl_info.blks_to_upgrd = csd->blks_to_upgrd;
 				jnl_info.free_blocks   = csd->trans_hist.free_blocks;
 				jnl_info.total_blks    = csd->trans_hist.total_blks;
-				GTMCRYPT_ONLY(
-					jnl_info_ptr = &jnl_info;
-					GTMCRYPT_COPY_HASH(csd, jnl_info_ptr);
-				)
+				jnl_info_ptr = &jnl_info;
+				GTMCRYPT_COPY_ENCRYPT_SETTINGS(csd, jnl_info_ptr);
                                 if (EXIT_NRM != (status = cre_jnl_file(&jnl_info)))
 				{	/* There was an error attempting to create the journal file */
 					exit_status |= status;
