@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2003, 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2003-2015 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -10,17 +11,11 @@
  ****************************************************************/
 
 #include "mdef.h"
-#ifdef VMS
-#include <ssdef.h>
-#include <descrip.h>
-#include <climsgdef.h>
-#include <jpidef.h>
-#include <fab.h>
-#include <rab.h>
-#include <nam.h>
-#include <rmsdef.h>
-#endif
+
 #include "gtm_unistd.h"
+#include "gtm_strings.h"
+
+#include "gtm_multi_thread.h"
 #include "gdsroot.h"
 #include "gtm_facility.h"
 #include "fileinfo.h"
@@ -57,19 +52,10 @@ error_def(ERR_MUPCLIERR);
 error_def(ERR_NOTPOSITIVE);
 error_def(ERR_RSYNCSTRMVAL);
 
-#ifdef VMS
-static	const	$DESCRIPTOR(output_qualifier,		"OUTPUT");
-static	const	$DESCRIPTOR(process_qualifier,		"PROCESS");
-#define EXCLUDE_CHAR	'-'
-#define STR2PID		asc_hex2i
-#define	MAX_PID_LEN	8	/* maximum number of hexadecimal digits in the process-id */
-#define REDIRECT_STR		"specify as (old-file-name=new-file-name,...)"
-#else
 #define EXCLUDE_CHAR	'~'
 #define STR2PID asc2i
 #define	MAX_PID_LEN	10	/* maximum number of decimal digits in the process-id */
 #define REDIRECT_STR		"specify as \"old-file-name=new-file-name,...\""
-#endif
 #define	WILDCARD_CHAR1	'*'
 #define	WILDCARD_CHAR2	'%'
 
@@ -92,31 +78,15 @@ void	mur_get_options(void)
 	char		*qual_buffer_ptr, *entry, *entry_ptr;
 	char		*file_name_specified, *file_name_expanded;
 	unsigned int 	file_name_specified_len, file_name_expanded_len;
-	int		extr_type, top, onln_rlbk_val;
+	int		extr_type, top, onln_rlbk_val, status2;
 	boolean_t	global_exclude;
 	long_list	*ll_ptr, *ll_ptr1;
 	redirect_list	*rl_ptr, *rl_ptr1, *tmp_rl_ptr;
 	select_list	*sl_ptr, *sl_ptr1;
 	boolean_t	interactive, parse_error;
-#	ifdef VMS
-	int4		item_code, mode;
-	jnl_proc_time	max_time;
-#	endif
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-#	ifdef VMS
-	DEBUG_ONLY(
-		JNL_WHOLE_TIME(max_time);
-		/* The following assert is to make sure we get some fix for the toggling of max_time's bit 55 (bit 0 lsb, 63 msb).
-		 * Bit 55 is currently 1 (it was 0 until some time in 1973) and will remain so until another 2087. Bit 55 toggles
-		 * approximately every 41700 days (114 years). We need to fix the way mupip recover operates to take care of such
-		 * transition periods. And the fix needs to be done before it is too late. Hence this assert. This assert will
-		 * fail in year 2084, approximately 980 days before the toggle time (in year 2087).
-		 */
-		assert(JNL_FULL_HI_TIME(max_time) < JNL_HITIME_WARN_THRESHOLD);
-	)
-#	endif
 	qual_buffer = (char *)malloc(MAX_LINE);
 	entry = (char *)malloc(MAX_LINE);
 	memset(&mur_options, 0, SIZEOF(mur_options));
@@ -128,49 +98,43 @@ void	mur_get_options(void)
 	mur_options.update = cli_present("RECOVER") == CLI_PRESENT;
 	/*----- 	-ROLLBACK	-----*/
 	mur_options.rollback = cli_present("ROLLBACK") == CLI_PRESENT;
-	UNIX_ONLY(assert(FALSE == jgbl.onlnrlbk);)
+	assert(FALSE == jgbl.onlnrlbk);
 	if (mur_options.rollback)
 	{
 		mur_options.update = TRUE;
-		UNIX_ONLY(
-			onln_rlbk_val = cli_present("ONLINE");
-			jgbl.onlnrlbk = onln_rlbk_val ? (onln_rlbk_val != CLI_NEGATED) : FALSE; /* Default is -NOONLINE */
-		)
+		onln_rlbk_val = cli_present("ONLINE");
+		jgbl.onlnrlbk = onln_rlbk_val ? (onln_rlbk_val != CLI_NEGATED) : FALSE; /* Default is -NOONLINE */
 	}
 	TREF(skip_file_corrupt_check) = mupip_jnl_recover = mur_options.update;
 	jgbl.mur_rollback = mur_options.rollback;	/* needed to set jfh->repl_state properly for newly created jnl files */
-	UNIX_ONLY(murgbl.resync_strm_index = INVALID_SUPPL_STRM;)
+	murgbl.resync_strm_index = INVALID_SUPPL_STRM;
 	if (CLI_PRESENT == cli_present("RESYNC"))
 	{
 		status = cli_get_uint64("RESYNC", (gtm_uint64_t *)&murgbl.resync_seqno);
 		if (!status || (0 == murgbl.resync_seqno))
 		{
-			gtm_putmsg(VARLSTCNT(4) ERR_NOTPOSITIVE, 2, LEN_AND_LIT("RESYNC"));
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_NOTPOSITIVE, 2, LEN_AND_LIT("RESYNC"));
 			mupip_exit(ERR_MUPCLIERR);
 		}
 		mur_options.resync_specified = TRUE;
-		UNIX_ONLY(
-			if (CLI_PRESENT == cli_present("RSYNC_STRM"))
+		if (CLI_PRESENT == cli_present("RSYNC_STRM"))
+		{
+			assert(CLI_PRESENT != cli_present("FORWARD"));
+			status = cli_get_int("RSYNC_STRM", &murgbl.resync_strm_index);
+			if (!status)
+				mupip_exit(ERR_MUPCLIERR);
+			if ((0 > murgbl.resync_strm_index) || (MAX_SUPPL_STRMS <= murgbl.resync_strm_index))
 			{
-				status = cli_get_int("RSYNC_STRM", &murgbl.resync_strm_index);
-				if (!status)
-					mupip_exit(ERR_MUPCLIERR);
-				if ((0 > murgbl.resync_strm_index) || (MAX_SUPPL_STRMS <= murgbl.resync_strm_index))
-				{
-					gtm_putmsg(VARLSTCNT(1) ERR_RSYNCSTRMVAL);
-					mupip_exit(ERR_MUPCLIERR);
-				}
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_RSYNCSTRMVAL);
+				mupip_exit(ERR_MUPCLIERR);
 			}
-		)
+		}
 	}
 	if ((status = cli_present("FETCHRESYNC")) == CLI_PRESENT)
 	{
 		if (!cli_get_int("FETCHRESYNC", &mur_options.fetchresync_port))
 			mupip_exit(ERR_MUPCLIERR);
 	}
-	/*-----		-[NO]VERIFY	-----*/
-	mur_options.verify = cli_present("VERIFY") != CLI_NEGATED;
-
 	/* SHOW[=(ALL|HEADER|PROCESSES|ACTIVE_PROCESSES|BROKEN_TRANSACTIONS|STATISTICS)] */
 	mur_options.show = SHOW_NONE;
 	if (CLI_PRESENT == cli_present("SHOW"))
@@ -199,6 +163,7 @@ void	mur_get_options(void)
 	for (extr_type = 0; extr_type < TOT_EXTR_TYPES; extr_type++)
 	{
 		mur_options.extr[extr_type] = FALSE;
+		mur_options.extr_fn_is_stdout[extr_type] = FALSE;
 		if (CLI_PRESENT == cli_present(extr_parms[extr_type]))
 		{
 			mur_options.extr[extr_type] = TRUE;
@@ -208,7 +173,9 @@ void	mur_get_options(void)
 				mur_options.extr_fn_len[extr_type] = length;
 				mur_options.extr_fn[extr_type] = (char *)malloc(mur_options.extr_fn_len[extr_type] + 1);
 				strncpy(mur_options.extr_fn[extr_type], qual_buffer, length);
-				mur_options.extr_fn[extr_type][length]='\0';
+				mur_options.extr_fn[extr_type][length] = '\0';
+				mur_options.extr_fn_is_stdout[extr_type] =
+					(0 == STRNCASECMP(qual_buffer, JNL_STDO_EXTR, SIZEOF(JNL_STDO_EXTR)));
 			}
 		}
 	}
@@ -218,6 +185,7 @@ void	mur_get_options(void)
 	/*-----		-BACKWARD		-----*/
 	assert(mur_options.forward != (cli_present("BACKWARD") == CLI_PRESENT));
 	DEBUG_ONLY(jgbl.mur_options_forward = mur_options.forward;)
+
 	/*----- JOURNAL TIME QUALIFIERS -----*/
 	/*-----		-AFTER=delta_or_absolute_time	-----*/
 	if (cli_present("AFTER") == CLI_PRESENT)
@@ -227,7 +195,7 @@ void	mur_get_options(void)
 		status = gtm_bintim(qual_buffer, &mur_options.after_time);
 		if (status)
 		{
-			gtm_putmsg(VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("AFTER"));
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("AFTER"));
 			mupip_exit(ERR_MUPCLIERR);
 		}
 	}
@@ -240,7 +208,7 @@ void	mur_get_options(void)
 		status = gtm_bintim(qual_buffer, &mur_options.since_time);
 		if (status)
 		{
-			gtm_putmsg(VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("SINCE"));
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("SINCE"));
 			mupip_exit(ERR_MUPCLIERR);
 		}
 		mur_options.since_time_specified = TRUE;
@@ -255,7 +223,7 @@ void	mur_get_options(void)
 		status = gtm_bintim(qual_buffer, &mur_options.before_time);
 		if (status)
 		{
-			gtm_putmsg(VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("BEFORE"));
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("BEFORE"));
 			mupip_exit(ERR_MUPCLIERR);
 		}
 		mur_options.before_time_specified = TRUE;
@@ -280,7 +248,7 @@ void	mur_get_options(void)
 			status = gtm_bintim(qual_buffer, &mur_options.lookback_time);
 			if (status)
 			{
-				gtm_putmsg(VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("TIME"));
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVQUALTIME, 2, LEN_AND_LIT("TIME"));
 				mupip_exit(ERR_MUPCLIERR);
 			}
 			mur_options.lookback_time_specified = TRUE;
@@ -309,7 +277,7 @@ void	mur_get_options(void)
 			/* parantheses are not allowed on UNIX */
 			if (('(' == *entry) || (')' == *(qual_buffer_ptr-1)))
 			{
-				gtm_putmsg(VARLSTCNT(4) ERR_INVREDIRQUAL, 2, LEN_AND_LIT(REDIRECT_STR));
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2, LEN_AND_LIT(REDIRECT_STR));
 				mupip_exit(ERR_MUPCLIERR);
 			}
 #endif
@@ -318,7 +286,7 @@ void	mur_get_options(void)
 				++cptr;
 			if ('=' != *cptr)
 			{
-				gtm_putmsg(VARLSTCNT(4) ERR_INVREDIRQUAL, 2, LEN_AND_LIT(REDIRECT_STR));
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2, LEN_AND_LIT(REDIRECT_STR));
 				mupip_exit(ERR_MUPCLIERR);
 			}
 			rl_ptr1 = (redirect_list *)malloc(SIZEOF(redirect_list));
@@ -334,7 +302,8 @@ void	mur_get_options(void)
 			if (!get_full_path(file_name_specified, file_name_specified_len, file_name_expanded,
 				&file_name_expanded_len, MAX_FN_LEN, &ustatus))
 			{
-				gtm_putmsg(VARLSTCNT(4) ERR_INVREDIRQUAL, 2, LEN_AND_LIT("Unable to find full pathname"));
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
+									LEN_AND_LIT("Unable to find full pathname"));
 				mupip_exit(ERR_MUPCLIERR);
 			}
 			for (tmp_rl_ptr = mur_options.redirect; tmp_rl_ptr != NULL; tmp_rl_ptr = tmp_rl_ptr->next)
@@ -344,7 +313,7 @@ void	mur_get_options(void)
 				    ((tmp_rl_ptr->new_name_len == file_name_expanded_len) &&
 				     (0 == memcmp(tmp_rl_ptr->new_name, file_name_expanded, tmp_rl_ptr->new_name_len))))
 				{
-					gtm_putmsg(VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
 						LEN_AND_LIT("Duplicate or invalid specification of files"));
 					mupip_exit(ERR_MUPCLIERR);
 				}
@@ -360,7 +329,8 @@ void	mur_get_options(void)
 			if (!get_full_path(file_name_specified, file_name_specified_len, file_name_expanded,
 				&file_name_expanded_len, MAX_FN_LEN, &ustatus))
 			{
-				gtm_putmsg(VARLSTCNT(4) ERR_INVREDIRQUAL, 2, LEN_AND_LIT("Unable to find full pathname"));
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
+										LEN_AND_LIT("Unable to find full pathname"));
 				mupip_exit(ERR_MUPCLIERR);
 			}
 			for (tmp_rl_ptr = mur_options.redirect; tmp_rl_ptr != NULL; tmp_rl_ptr = tmp_rl_ptr->next)
@@ -370,7 +340,7 @@ void	mur_get_options(void)
 				   (tmp_rl_ptr->new_name_len == file_name_expanded_len &&
 				    0 == memcmp(tmp_rl_ptr->new_name, file_name_expanded, tmp_rl_ptr->new_name_len)))
 				{
-					gtm_putmsg(VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
 						LEN_AND_LIT("Duplicate or invalid specification of files"));
 					mupip_exit(ERR_MUPCLIERR);
 				}
@@ -384,25 +354,22 @@ void	mur_get_options(void)
 		free(file_name_expanded);
 	}
 	/*----- 	-FENCES=NONE|ALWAYS|PROCESS 	-----*/
-	mur_options.fences = FENCE_PROCESS;
+	mur_options.fences = FENCE_PROCESS;	/* DEFAULT */
 	if (cli_present("FENCES") == CLI_PRESENT)
 	{
 		if (CLI_PRESENT == cli_present("FENCES.NONE"))
 			mur_options.fences = FENCE_NONE;
 		else if (CLI_PRESENT == cli_present("FENCES.ALWAYS"))
 			mur_options.fences = FENCE_ALWAYS;
-		else if (CLI_PRESENT == cli_present("FENCES.PROCESS"))
-			mur_options.fences = FENCE_PROCESS;	/* DEFAULT */
+		if (mur_options.rollback && (FENCE_PROCESS != mur_options.fences))
+		{
+			util_out_print("MUPIP JOURNAL -ROLLBACK only supports -FENCES=PROCESS", TRUE);
+			mupip_exit(ERR_MUPCLIERR);
+		}
 	}
 	DEBUG_ONLY(jgbl.mur_fences_none = (FENCE_NONE == mur_options.fences);)
 	/*-----		-[NO]INTERACTIVE	-----*/
-#ifdef VMS
-	item_code = JPI$_MODE;
-	lib$getjpi(&item_code, NULL, NULL, &mode, NULL, NULL);
-	interactive = (JPI$K_INTERACTIVE == mode);
-#else
 	interactive = (boolean_t) isatty(0);
-#endif
 	mur_options.interactive = interactive && (CLI_NEGATED != cli_present("INTERACTIVE"));
 	/*-----		-[NO]CHAIN 		-----*/
 	mur_options.chain = TRUE; /* By Default or specified without negation */
@@ -415,18 +382,18 @@ void	mur_get_options(void)
 			mupip_exit(ERR_MUPCLIERR);
 		if (mur_options.error_limit < 0)
 		{
-			gtm_putmsg(VARLSTCNT(2) ERR_INVERRORLIM, 0);
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(2) ERR_INVERRORLIM, 0);
 			mupip_exit(ERR_MUPCLIERR);
 		}
 	} else if (status == CLI_NEGATED)
 		mur_options.error_limit = 1000000;
-#ifdef VMS
-	/*-----		-OUTPUT (VMS ONLY) 	-----*/
-	if ( CLI_PRESENT == cli_present("OUTPUT"))
-		util_out_open(&output_qualifier);
-#endif
 	/*-----		-[NO]CHECKTN 		-----*/
 	mur_options.notncheck = (cli_present("CHECKTN") == CLI_NEGATED);
+	if (mur_options.notncheck && mur_options.rollback && mur_options.forward)
+	{
+		util_out_print("MUPIP JOURNAL -ROLLBACK -FORWARD does not support -NOCHECKTN", TRUE);
+		mupip_exit(ERR_MUPCLIERR);
+	}
 	/*----- JOURNAL SELECTION QUALIFIERS -----*/
 	/*-----		-GLOBAL=(list of global names)	-----*/
 	if (cli_present("GLOBAL") == CLI_PRESENT)
@@ -502,8 +469,8 @@ void	mur_get_options(void)
 				*entry_ptr++ = *qual_buffer_ptr++;
 				if (parse_error || ((qual_buffer_ptr == ctop) && (state > 1)))
 				{
-					gtm_putmsg(VARLSTCNT(5) ERR_INVGLOBALQUAL, 3, qual_buffer_ptr - &qual_buffer[0],
-								ctop - &qual_buffer[0], qual_buffer);
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_INVGLOBALQUAL, 3,
+						qual_buffer_ptr - &qual_buffer[0], ctop - &qual_buffer[0], qual_buffer);
 					mupip_exit(ERR_MUPCLIERR);
 				}
 			}
@@ -579,7 +546,7 @@ void	mur_get_options(void)
 			qual_buffer_ptr[--length] = '\0';
 		for (ctop = qual_buffer_ptr + length; qual_buffer_ptr < ctop;)
 		{
-			if (!cli_get_str_ele(qual_buffer_ptr, entry, &length, UNIX_ONLY(FALSE) VMS_ONLY(TRUE)))
+			if (!cli_get_str_ele(qual_buffer_ptr, entry, &length, FALSE))
 				mupip_exit(ERR_MUPCLIERR);
 			qual_buffer_ptr += length;
 			assert(',' == *qual_buffer_ptr || !(*qual_buffer_ptr));	/* either comma separator or end of option list */
@@ -653,7 +620,7 @@ void	mur_get_options(void)
 			qual_buffer_ptr[--length] = '\0';
 		for (ctop = qual_buffer_ptr + length; qual_buffer_ptr < ctop;)
 		{
-			if (!cli_get_str_ele(qual_buffer_ptr, entry, &length, UNIX_ONLY(FALSE) VMS_ONLY(TRUE)))
+			if (!cli_get_str_ele(qual_buffer_ptr, entry, &length, FALSE))
 				mupip_exit(ERR_MUPCLIERR);
 			qual_buffer_ptr += length;
 			assert(',' == *qual_buffer_ptr || !(*qual_buffer_ptr));	/* either comma separator or end of option list */
@@ -686,7 +653,7 @@ void	mur_get_options(void)
 			if ((MAX_PID_LEN < length) ||
 			    ((ll_ptr->num = STR2PID((uchar_ptr_t)entry_ptr, length)) == (unsigned int) - 1))
 			{
-				gtm_putmsg(VARLSTCNT(4) ERR_INVIDQUAL, 2, length, entry_ptr);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVIDQUAL, 2, length, entry_ptr);
 				mupip_exit(ERR_MUPCLIERR);
 			}
 		}
@@ -707,97 +674,48 @@ void	mur_get_options(void)
 			mur_options.transaction |= TRANS_KILLS;
 		if ((TRANS_KILLS != mur_options.transaction) && (TRANS_SETS != mur_options.transaction))
 		{
-			gtm_putmsg(VARLSTCNT(2) ERR_INVTRNSQUAL, 0);
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(2) ERR_INVTRNSQUAL, 0);
 			mupip_exit(ERR_MUPCLIERR);
 		}
 		mur_options.selection = TRUE;
 	}
-#ifdef VMS
-	/*----- 	/PROCESS=(list of user process names) 	-----*/
-	if (cli_present("PROCESS") == CLI_PRESENT)
-	{	/* this is VMS only */
-		length = MAX_LINE;
-		if (!CLI_GET_STR_ALL("PROCESS", qual_buffer, &length))
-			mupip_exit(ERR_MUPCLIERR);
-		qual_buffer_ptr = qual_buffer;
-		global_exclude = FALSE;
-		if ('"' == *qual_buffer_ptr )
-		{
-			++qual_buffer_ptr;
-			--length;
-			if ('"' == qual_buffer_ptr[length-1])
-				qual_buffer_ptr[--length] = '\0';
-		}
-		if (EXCLUDE_CHAR == *qual_buffer_ptr)
-		{
-			global_exclude = TRUE;
-			++qual_buffer_ptr;
-			--length;
-		}
-		if ('(' == *qual_buffer_ptr )
-		{
-			++qual_buffer_ptr;
-			--length;
-		} else if (global_exclude)
-		{
-			--qual_buffer_ptr;
-			++length;
-			global_exclude = FALSE;
-		}
-		if (')' == qual_buffer_ptr[length-1])
-			qual_buffer_ptr[--length] = '\0';
-		for (ctop = qual_buffer_ptr + length; qual_buffer_ptr < ctop;)
-		{
-			if (!cli_get_str_ele(qual_buffer_ptr, entry, &length, FALSE))
-				mupip_exit(ERR_MUPCLIERR);
-			qual_buffer_ptr += length;
-			assert((',' == *qual_buffer_ptr) || !(*qual_buffer_ptr)); /* either comma separator or end of option list */
-			if (',' == *qual_buffer_ptr)
-				qual_buffer_ptr++;  /* skip separator */
-			entry_ptr = entry;
-			sl_ptr1 = (select_list *)malloc(SIZEOF(select_list));
-			sl_ptr1->next = NULL;
-			if (mur_options.process == NULL)
-				mur_options.process = sl_ptr1;
-			else
-				sl_ptr->next = sl_ptr1;
-			sl_ptr = sl_ptr1;
-			if ('"' == entry_ptr[length - 1])
-				--length;
-			if ('"' == *entry_ptr)
-			{
-				entry_ptr++;
-				length--;
-			}
-			if (EXCLUDE_CHAR == *entry_ptr)
-			{
-				++entry_ptr;
-				--length;
-				sl_ptr->exclude = TRUE;
-			} else
-				sl_ptr->exclude = FALSE;
-			if (global_exclude)
-				sl_ptr->exclude = !sl_ptr->exclude;
-			sl_ptr->len = length;
-			sl_ptr->buff = (char *)malloc(length);
-			memcpy(sl_ptr->buff, entry_ptr, length);
-			sl_ptr->has_wildcard = FALSE;
-			sl_ptr->has_wildcard += ((NULL == memchr(sl_ptr->buff, WILDCARD_CHAR1, length)) ? FALSE : TRUE);
-			sl_ptr->has_wildcard += ((NULL == memchr(sl_ptr->buff, WILDCARD_CHAR2, length)) ? FALSE : TRUE);
-		}
-		mur_options.selection = TRUE;
+	/*-----		-[NO]VERIFY	-----*/
+	/* -VERIFY is default except in case RECOVER -FORWARD or ROLLBACK -FORWARD where -NOVERIFY is default.
+	 * Also, if -NOCHECKTN is specified, we want -VERIFY to be default so we get the
+	 * "Transaction number continuity check failed" message. Additionally, not doing so could cause tp_resolve_time
+	 * to be set to a non-zero value (in mur_tp_resolve_time.c) and in turn cause issues in forward processing phase
+	 * due to discontinuous timeranges in consecutive jnl file generations like what v54003/C9K08003315 subtest induces.
+	 */
+	assert(FALSE == mur_options.verify_specified);
+	if (CLI_PRESENT == (status = cli_present("VERIFY")))
+	{
+		mur_options.verify = TRUE;
+		mur_options.verify_specified = TRUE;
 	}
-#endif
+	else if (CLI_NEGATED == status)
+		mur_options.verify = FALSE;
+	else
+		mur_options.verify = (!mur_options.update || !mur_options.forward || mur_options.notncheck);
+
 	/* by default after_images are applied during backward recovery. the APPLY_AFTER_IMAGE option can override that behaviour */
 	mur_options.apply_after_image = !mur_options.forward;
 	if (CLI_PRESENT == (status = cli_present("APPLY_AFTER_IMAGE")))
 		mur_options.apply_after_image = TRUE;
 	else if (CLI_NEGATED == status)
 		mur_options.apply_after_image = FALSE;
-	/* if the only request is -SHOW=HEAD, set show_head_only */
-	if ((SHOW_HEADER == mur_options.show) && !mur_options.update && !mur_options.verify &&
-	    (CLI_PRESENT != cli_present("EXTRACT")))
+	/* If the only request is -SHOW=HEAD, set show_head_only. Also reset -VERIFY if previously assumed by default */
+	if ((SHOW_HEADER == mur_options.show) && !mur_options.update
+			&& !mur_options.verify_specified && (CLI_PRESENT != cli_present("EXTRACT")))
+	{
 		mur_options.show_head_only = TRUE;
+		mur_options.verify = FALSE;
+	}
+	/*-----		-PARALLEL=MAXTHREADS/MAXPROCS	-----*/
+	if (CLI_PRESENT == cli_present("PARALLEL"))
+	{
+		if (!cli_get_int("PARALLEL", &gtm_mupjnl_parallel))
+			gtm_mupjnl_parallel = 0; /* Treat -PARALLEL without any value as full parallelism */
+	}
 	free(entry);
 	free(qual_buffer);
 }
