@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2015 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -394,7 +395,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 				n = (NULL != gname) ? gname->act : 0;
 			} else
 				n = 0;
-			gvkey = &save_currkey[0];
+			gvkey = (gv_key *)save_currkey;
 			key = gvn2gds(arg1, gvkey, n);
 			assert(key > &gvkey->base[0]);
 			assert(gvkey->end == key - &gvkey->base[0] - 1);
@@ -715,6 +716,9 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_LOGTPRESTART:
 			n = TREF(tprestart_syslog_delta);
 			break;
+		case VTK_LOGNONTP:
+			n = TREF(nontprestart_log_delta);
+			break;
 #		ifdef UNIX
 		case VTK_JNLERROR:
 			n = TREF(error_on_jnl_file_lost);
