@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -75,6 +75,7 @@
 #endif
 #include "gtmimagename.h"
 #include "cache.h"
+#include "set_gbuff_limit.h"	/* Needed for set_gbuff_limit() */
 
 STATICFNDCL void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg);
 
@@ -258,22 +259,6 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 			if (old_bool != TREF(gtm_fullbool))
 				cache_table_rebuild();
 			break;
-		case VTK_GDSCERT0:
-			outval.mvtype = MV_STR;
-			op_wteol(1);
-			WRITE_LITERAL(msg1);
-			certify_all_blocks = FALSE;
-			WRITE_LITERAL(msg2);
-			op_wteol(1);
-			break;
-		case VTK_GDSCERT1:
-			outval.mvtype = MV_STR;
-			op_wteol(1);
-			WRITE_LITERAL(msg1);
-			certify_all_blocks = TRUE;
-			WRITE_LITERAL(msg3);
-			op_wteol(1);
-			break;
 		case VTK_GDSCERT:
 			outval.mvtype = MV_STR;
 			op_wteol(1);
@@ -722,8 +707,7 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 			{
 				if (!TREF(tprestart_syslog_delta))
 					TREF(tprestart_syslog_delta) = 1;
-			}
-			else
+			} else
 			{
 				TREF(tprestart_syslog_delta) = MV_FORCE_INT(parmblk.value);
 				if (0 > TREF(tprestart_syslog_delta))
@@ -734,6 +718,22 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 		case VTK_NOLOGTPRESTART:
 			TREF(tprestart_syslog_delta) = 0;
 			break;
+		case VTK_LOGNONTP:
+			if (!numarg)
+			{
+				if (!TREF(nontprestart_log_delta))
+					TREF(nontprestart_log_delta) = 1;
+			} else
+			{
+				TREF(nontprestart_log_delta) = MV_FORCE_INT(parmblk.value);
+				if (0 > TREF(nontprestart_log_delta))
+					TREF(nontprestart_log_delta) = 0;
+			}
+			TREF(nontprestart_count) = 0;
+			break;
+		case VTK_NOLOGNONTP:
+			TREF(nontprestart_log_delta) = 0;
+			break;
 #		ifdef UNIX
 		case VTK_LINK:
 			init_relink_allowed(&parmblk.value->str);
@@ -916,14 +916,7 @@ void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 			case VTK_POOLLIMIT:
 				csa = cs_addrs;
 				csd = csa->hdr;
-				nbuffs = ((dba_bg == csd->acc_meth) && (NULL != thirdarg)) ? MV_FORCE_INT(thirdarg) : 0;
-				if (nbuffs && (MV_STR & thirdarg->mvtype) && ('%' == thirdarg->str.addr[thirdarg->str.len - 1]))
-					nbuffs = (100 == nbuffs) ? 0 : (csd->n_bts * nbuffs) / 100;		/* Percentage */
-				csa->gbuff_limit = (0 == nbuffs) ? 0 : MAX(MIN(nbuffs, csd->n_bts * .5), MIN_GBUFF_LIMIT);
-				/* To pick the current "hand" as a pseudo-random spot for our area see dbg code in gvcst_init
-				 * but for the first release of this always pick the end of the buffer
-				 */
-				csa->our_midnite = csa->acc_meth.bg.cache_state->cache_array + csd->bt_buckets + csd->n_bts;
+				set_gbuff_limit(&csa, &csd, thirdarg);
 				break;
 		}
 	}
