@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -120,6 +120,7 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 	socket_interrupt 	*sockintr;
 	socket_struct		*socketptr;
 	zintcmd_ops		zintcmd_command;
+	intrpt_state_t		prev_intrpt_state;
 	UNIX_ONLY(d_tt_struct	*tt_ptr;)
 	DBGRFCT_ONLY(mident_fixed vname;)
 	DCL_THREADGBL_ACCESS;
@@ -151,10 +152,8 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 					tptimeout_set(0);
 			} else if (mv_st_ent->mv_st_cont.mvs_msav.addr == &dollar_zgbldir)
 			{
-				if (0 != dollar_zgbldir.str.len)
-					gd_header = zgbldir(&dollar_zgbldir);
-				else
-					dpzgbini();	/* sets gd_header to NULL */
+				/* Restore GLD if a match is found, otherwise defer setting gd_header */
+				gd_header = zgbldir_name_lookup_only(&dollar_zgbldir);
 				if (gv_currkey)
 				{
 					gv_currkey->base[0] = 0;
@@ -464,7 +463,7 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 				dollar_ztrap = mv_st_ent->mv_st_cont.mvs_trigr.dollar_ztrap_save;
 				ztrap_explicit_null = mv_st_ent->mv_st_cont.mvs_trigr.ztrap_explicit_null_save;
 			}
-			DEFER_INTERRUPTS(INTRPT_IN_CONDSTK);
+			DEFER_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state);
 			CHECKHIGHBOUND(mv_st_ent->mv_st_cont.mvs_trigr.ctxt_save);
 			CHECKLOWBOUND(mv_st_ent->mv_st_cont.mvs_trigr.ctxt_save);
 			ctxt = mv_st_ent->mv_st_cont.mvs_trigr.ctxt_save;
@@ -481,7 +480,7 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 								|| (&gvcst_spr_kill_ch == ctxt->ch)))))));
 			active_ch = ctxt;
 			ctxt->ch_active = FALSE;
-			ENABLE_INTERRUPTS(INTRPT_IN_CONDSTK);
+			ENABLE_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state);
 			if (tp_timeout_deferred && !((0 < dollar_ecode.index) && (ETRAP_IN_EFFECT))
 			    && !dollar_zininterrupt)
 			{	/* A tp timeout was deferred. Now that $ETRAP is no longer in effect and/or we are no
