@@ -33,6 +33,7 @@
 # 2011-05-03  0.11 K.S. Bhaskar - Allow for letter suffix releases
 # 2011-10-25  0.12 K.S. Bhaskar - Support option to delete .o files on shared library platforms
 # 2014-08-13  0.13 K.S. Bhaskar - Add verbosity around getting latest version and tarball, if requested
+# 2015-10-13  0.14 GT.M Staff   - Fix a few minor bugs
 
 # Turn on debugging if set
 if [ "Y" = "$gtm_debug" ] ; then set -x ; fi
@@ -426,7 +427,6 @@ if [ "Y" = "$gtm_prompt_for_group" -o 54002 -le `echo $gtm_version | cut -s -d V
 fi
 if [ "N" = "$gtm_group_already" ] ; then
     echo $gtm_group_restriction >>$gtm_configure_in
-    if [ "Y" = "$gtm_group_restriction" ] ; then echo $gtm_group >>$gtm_configure_in ; fi
 fi
 echo $gtm_installdir >>$gtm_configure_in
 echo y >>$gtm_configure_in
@@ -467,7 +467,7 @@ if [ -d "$gtm_linkenv" ] ; then
     ( cd $gtm_linkenv ; ln -s $gtm_installdir/gtmprofile $gtm_installdir/gtmcshrc ./ )
     if [ "Y" = "$gtm_verbose" ] ; then echo Linked env ; ls -l $gtm_linkenv ; fi
 else if [ -d "$gtm_copyenv" ] ; then
-        ( cd $gtm_linkenv ; cp $gtm_installdir/gtmprofile $gtm_installdir/gtmcshrc ./ )
+        ( cd $gtm_copyenv ; cp $gtm_installdir/gtmprofile $gtm_installdir/gtmcshrc ./ )
         if [ "Y" = "$gtm_verbose" ] ; then echo Copied env ; ls -l $gtm_copyenv ; fi
      fi
 fi
@@ -475,7 +475,7 @@ if [ -d "$gtm_linkexec" ] ; then
     ( cd $gtm_linkexec ; ln -s $gtm_installdir/gtm ./ )
     if [ "Y" = "$gtm_verbose" ] ; then echo Linked exec ; ls -l $gtm_linkexec ; fi
 else if [ -d "$gtm_copyexec" ] ; then
-        ( cd $gtm_linkexec ; cp $gtm_installdir/gtm ./ )
+        ( cd $gtm_copyexec ; cp $gtm_installdir/gtm ./ )
         if [ "Y" = "$gtm_verbose" ] ; then echo Copied exec ; ls -l $gtm_copyexec ; fi
      fi
 fi
